/****************************************************************
Copyright (c) 1998 Sandro Sigala <ssigala@globalnet.it>.
All rights reserved.

Permission to use, copy, modify, and distribute this software
and its documentation for any purpose and without fee is hereby
granted, provided that the above copyright notice appear in all
copies and that both that the copyright notice and this
permission notice and warranty disclaimer appear in supporting
documentation, and that the name of the author not be used in
advertising or publicity pertaining to distribution of the
software without specific, written prior permission.

The author disclaim all warranties with regard to this
software, including all implied warranties of merchantability
and fitness.  In no event shall the author be liable for any
special, indirect or consequential damages or any damages
whatsoever resulting from loss of use, data or profits, whether
in an action of contract, negligence or other tortious action,
arising out of or in connection with the use or performance of
this software.
****************************************************************/

#include "config.h"

#include <qlabel.h>
#include <qlistbox.h>
#include <qpainter.h>
#include <qpushbt.h>
#include <kcolorbtn.h>
#include <qtooltip.h>

#include "kreptoned.h"
#include "spritewindow.h"
#include "spritepreviewframe.h"
#include "spritegridframe.h"

SpriteWindow::SpriteWindow(QPixmap *xsprites, QWidget *parent, const char *name)
	: QWindow(parent, name)
{
	const char *sprite_names[] = {
#define SPRITEOBJ(e, n) n,
#define MAPOBJ(e, n) n,
#include "sprites.h"
#undef SPRITEOBJ
#undef MAPOBJ
		0
	};

	sprites = xsprites;
	setCaption("Sprite Editor");
	setFixedSize(525, 305);

	sprite_list = new QListBox(this);
	sprite_list->setGeometry(10, 30, 150, 210);
	sprite_list->setAutoUpdate(false);
	for (const char **s = &sprite_names[0]; *s != 0; ++s)
		sprite_list->insertItem(*s);
	sprite_list->setAutoUpdate(true);
	sprite_list->repaint();
	QToolTip::add(sprite_list, "The list of available sprites to edit");

	QLabel *label = new QLabel(sprite_list, "&Sprite list", this);
	label->setAutoResize(true);
	label->move(10, 10);

	preview_sprite = new SpritePreviewFrame(sprites, this);
	preview_sprite->move(185, 30);
	QToolTip::add(preview_sprite, "The currently edited sprite");
	label = new QLabel(preview_sprite, "Preview", this);
	label->setAutoResize(true);
	label->move(185, 10);

	left_color = new KColorButton(this);
	left_color->setGeometry(170, 100, 30, 30);
	left_color->setColor(white);
	QToolTip::add(left_color, "The color of the left button of the mouse");
	label = new QLabel(left_color, "&Left", this);
	label->setAutoResize(true);
	label->move(170, 80);

	right_color = new KColorButton(this);
	right_color->setGeometry(210, 100, 30, 30);
	right_color->setColor(black);
	QToolTip::add(right_color, "The color of the right button of the mouse");
	label = new QLabel(right_color, "&Right", this);
	label->setAutoResize(true);
	label->move(210, 80);

	grid_sprite = new SpriteGridFrame(sprites, this);
	grid_sprite->move(250, 30);
	label = new QLabel(grid_sprite, "Sprite", this);
	label->setAutoResize(true);
	label->move(250, 10);

	close_button = new QPushButton("&Close", this);
	close_button->setGeometry(10, 260, 70, 30);
	close_button->setDefault(true);
	QToolTip::add(close_button, "Close this window");

	coord = new QLabel("X:1 Y:1", this);
	coord->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	coord->setAutoResize(true);
	coord->move(grid_sprite->x() + grid_sprite->width() - coord->width(), 3);

        sprite_list->setFocus();

	connect(close_button, SIGNAL(clicked()),
		this, SLOT(hide()));
	connect(sprite_list, SIGNAL(highlighted(int)),
		this, SLOT(selectedSprite(int)));
	connect(grid_sprite, SIGNAL(pressedButton(int, int, int)),
		this, SLOT(pressedButton(int, int, int)));
	connect(grid_sprite, SIGNAL(changedCoordinates(int, int)),
		this, SLOT(updateCoordinates(int, int)));
}

void SpriteWindow::selectedSprite(int i)
{
	current_sprite = (Sprite)i;
	updateChilds();
}

void SpriteWindow::pressedButton(int button, int x, int y)
{
	QPainter p;
	p.begin(&sprites[current_sprite]);
	if (button & LeftButton)
		p.setPen(left_color->color());
	else if (button & RightButton)
		p.setPen(right_color->color());
	p.drawLine(x, y, x, y);
	p.end();
	emit changedSprite();
}

void SpriteWindow::updateCoordinates(int x, int y)
{
	QString s;
	s.sprintf("X:%d Y:%d", x+1, y+1);
	coord->setText(s);
	coord->move(grid_sprite->x() + grid_sprite->width() - coord->width(), 3);
}

void SpriteWindow::updateChilds()
{
	preview_sprite->setSprite(current_sprite);
	preview_sprite->repaint(false);
	grid_sprite->setSprite(current_sprite);
	grid_sprite->repaint(false);	
}
